package org.processmining.subtrace.dialogs;

import java.awt.*;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.KeyEvent;
import java.awt.event.KeyListener;
import java.util.*;
import java.util.List;

import javax.swing.*;
import javax.swing.border.Border;

import com.fluxicon.slickerbox.components.RoundedPanel;
import com.fluxicon.slickerbox.ui.SlickerScrollBarUI;
import org.deckfour.xes.classification.XEventClass;
import org.deckfour.xes.classification.XEventClasses;
import org.deckfour.xes.info.XLogInfo;
import org.deckfour.xes.model.XLog;
import org.processmining.contexts.uitopia.UIPluginContext;
import org.processmining.framework.util.collection.AlphanumComparator;
import org.processmining.framework.util.ui.widgets.ProMTextField;
import org.processmining.framework.util.ui.widgets.WidgetColors;
import org.processmining.subtrace.parameters.SubTraceFilteringParameters;

import com.fluxicon.slickerbox.factory.SlickerFactory;

import info.clearthought.layout.TableLayout;
import info.clearthought.layout.TableLayoutConstants;

public abstract class SubTraceDialog extends JPanel {

	/**
	 * 
	 */
	private static final long serialVersionUID = -60087716353524468L;
    protected final XLog incoming;
    protected final XLogInfo logInfo;

    private JComponent list;


	/**
	 * The JPanel that allows the user to set (a subset of) the parameters.
	 */
	public SubTraceDialog(UIPluginContext context, XLog input,
						  final SubTraceFilteringParameters parameters,
                          XLogInfo logInfo) {

        this.incoming = input;
        this.logInfo = logInfo;


        double size[][] = {{80, TableLayoutConstants.FILL}, {TableLayoutConstants.FILL, 30, 30, 30, 30, 30}};
        setLayout(new TableLayout(size));

        JComponent outputName = generateOutputName(parameters);
        if (outputName != null) {
            add(new JLabel("Log name:"), "0, 5");
            this.add(outputName, "1, 5");
        }

        list = generateList(parameters);
        if (list != null) {
            this.add(list, "0, 0, 1, 0");
        }

        JComponent range = generateRange(parameters);
        if (range != null) {
            this.add(range, "0, 1, 1, 1");
        }

        JComponent position = generatePosition(parameters);
        if (position != null) {
            this.add(position, "0, 2, 1, 2");
        }

        JComponent extraEvent = generateExtraEvent(parameters);
        if (extraEvent != null) {
            this.add(extraEvent, "0, 3, 1, 3");
        }

        JComponent extraName = generateExtraName(parameters);
        if (extraName != null) {
            this.add(extraName, "0, 4, 1, 4");
        }

    }

    public void refreshDetails(SubTraceFilteringParameters parameters) {
        list = generateList(parameters);
        if (list != null) {
            this.add(list, "0, 0, 1, 0");
        }
        this.validate();
        this.repaint();
    }

    // create event/classifier selection list
    protected abstract JComponent generateList(SubTraceFilteringParameters parameters);
    // generate RANGE selection dialog
    protected abstract JComponent generateRange(SubTraceFilteringParameters parameters);

    protected abstract JComponent generatePosition(SubTraceFilteringParameters parameters);

    protected abstract JComponent generateExtraEvent(SubTraceFilteringParameters parameters);

    protected abstract JComponent generateExtraName(SubTraceFilteringParameters parameters);

    protected abstract JComponent generateOutputName(SubTraceFilteringParameters parameters);



    protected Color colorListBg = new Color(60, 60, 60);
    protected Color colorListBgSelected = new Color(10, 90, 10);
    protected Color colorListFg = new Color(200, 200, 200, 160);
    protected Color colorListFgSelected = new Color(230, 230, 230, 200);
    protected Color colorListEnclosureBg = new Color(150, 150, 150);
    protected Color colorListHeader = new Color(10, 10, 10);
    protected Color colorListDescription = new Color(60, 60, 60);

    protected JComponent configureList(JList<Object> list, String title, String description, boolean multiple) {
        list.setBackground(colorListBg);
        list.setForeground(colorListFg);
        list.setSelectionBackground(colorListBgSelected);
        list.setSelectionForeground(colorListFgSelected);
        list.setFont(list.getFont().deriveFont(12f));
        list.setSelectionMode(multiple?ListSelectionModel.MULTIPLE_INTERVAL_SELECTION:ListSelectionModel.SINGLE_SELECTION);
        return configureAnyScrollable(list, title, description);
    }

    protected JComponent configureAnyScrollable(JComponent scrollable, String title, String description) {
        RoundedPanel enclosure = new RoundedPanel(10, 5, 5);
        enclosure.setBackground(colorListEnclosureBg);
        enclosure.setLayout(new BoxLayout(enclosure, BoxLayout.Y_AXIS));
        JLabel headerLabel = new JLabel(title);
        headerLabel.setOpaque(false);
        headerLabel.setForeground(colorListHeader);
        headerLabel.setFont(headerLabel.getFont().deriveFont(14f));
        JLabel descriptionLabel = new JLabel("<html>" + description + "</html>");
        descriptionLabel.setOpaque(false);
        descriptionLabel.setForeground(colorListDescription);
        descriptionLabel.setFont(descriptionLabel.getFont().deriveFont(11f));
        JScrollPane listScrollPane = new JScrollPane(scrollable);
        listScrollPane.setVerticalScrollBarPolicy(ScrollPaneConstants.VERTICAL_SCROLLBAR_AS_NEEDED);
        listScrollPane.setHorizontalScrollBarPolicy(ScrollPaneConstants.HORIZONTAL_SCROLLBAR_NEVER);
        listScrollPane.setViewportBorder(BorderFactory.createLineBorder(new Color(40, 40, 40)));
        listScrollPane.setBorder(BorderFactory.createEmptyBorder());
        JScrollBar vBar = listScrollPane.getVerticalScrollBar();
        vBar.setUI(new SlickerScrollBarUI(vBar, colorListEnclosureBg, new Color(30, 30, 30), new Color(80, 80, 80), 4, 12));
        enclosure.add(packLeftAligned(headerLabel));
        enclosure.add(Box.createVerticalStrut(3));
        enclosure.add(packLeftAligned(descriptionLabel));
        enclosure.add(Box.createVerticalStrut(5));
        enclosure.add(listScrollPane);
        return enclosure;
    }

    protected JComponent packLeftAligned(JComponent component) {
        JPanel packed = new JPanel();
        packed.setOpaque(false);
        packed.setBorder(BorderFactory.createEmptyBorder());
        packed.setLayout(new BoxLayout(packed, BoxLayout.X_AXIS));
        packed.add(component);
        packed.add(Box.createHorizontalGlue());
        return packed;
    }

    protected JComponent configurePanel(List<JRadioButton> buttons, String title) {
        //Put the radio buttons in a column in a panel.
        JPanel radioPanel = new JPanel();
        radioPanel.setBorder(BorderFactory.createEmptyBorder(0, 0, 0, 0));
        radioPanel.setLayout(new BoxLayout(radioPanel, BoxLayout.LINE_AXIS));
        radioPanel.setOpaque(false);
        JLabel headerLabel = new JLabel(title);
        headerLabel.setOpaque(false);
        headerLabel.setForeground(colorListHeader);
        headerLabel.setFont(headerLabel.getFont().deriveFont(11f));

        radioPanel.add(headerLabel);
        radioPanel.add(Box.createRigidArea(new Dimension(15, 0)));

        ButtonGroup group = new ButtonGroup();
        for (JRadioButton button: buttons) {
            button.setOpaque(false);
            button.setForeground(colorListDescription);
            button.setFocusPainted(false);
            button.setFont(button.getFont().deriveFont(11f));
            group.add(button);
            radioPanel.add(button);
            radioPanel.add(Box.createRigidArea(new Dimension(5, 0)));
        }

        return radioPanel;
    }

    protected JComponent configureTextField(JTextField textField, String label) {
//        JPanel textPanel = new JPanel(new GridLayout(1, 0));
        JPanel textPanel = new JPanel();
        textPanel.setBorder(BorderFactory.createEmptyBorder(0, 0, 0, 10));
        textPanel.setLayout(new BoxLayout(textPanel, BoxLayout.LINE_AXIS));
        textPanel.setOpaque(false);
        textPanel.setAlignmentX(LEFT_ALIGNMENT);

        JLabel fieldLabel = new JLabel(label);
        fieldLabel.setOpaque(false);
        fieldLabel.setForeground(colorListHeader);
        fieldLabel.setFont(fieldLabel.getFont().deriveFont(11f));
        fieldLabel.setHorizontalAlignment(SwingConstants.LEFT);
        textPanel.add(fieldLabel);

        textPanel.add(Box.createRigidArea(new Dimension(15, 0)));

        textField.setBorder(null);
        textField.setOpaque(true);
        textField.setBorder(BorderFactory.createEmptyBorder(2, 5, 2, 5));
        textField.setBackground(WidgetColors.COLOR_LIST_BG);
        textField.setForeground(WidgetColors.COLOR_LIST_SELECTION_FG);
        textField.setSelectionColor(WidgetColors.COLOR_LIST_SELECTION_BG);
        textField.setSelectedTextColor(WidgetColors.COLOR_LIST_SELECTION_FG);
        textField.setCaretColor(WidgetColors.COLOR_LIST_SELECTION_FG);
        textField.setFont(textField.getFont().deriveFont(11f));
        textField.setSize((int)Math.round(textField.getPreferredSize().getWidth()), 15);

        textPanel.add(textField);
        //textPanel.add(Box.createHorizontalGlue());

        return textPanel;
    }

    /**
     * Sort events alphanumerically
     */
    protected class EventClassComparator implements Comparator<XEventClass> {
        public int compare(XEventClass o1, XEventClass o2) {
            return (new AlphanumComparator().compare(o1.toString(), o2.toString()));
        }
    }
}
