package org.processmining.subtrace.dialogs;

import org.apache.commons.lang3.ArrayUtils;
import org.deckfour.xes.classification.XEventClass;
import org.deckfour.xes.classification.XEventClasses;
import org.deckfour.xes.classification.XEventClassifier;
import org.deckfour.xes.info.XLogInfo;
import org.deckfour.xes.info.impl.XLogInfoImpl;
import org.deckfour.xes.model.XLog;
import org.processmining.contexts.uitopia.UIPluginContext;
import org.processmining.framework.util.ui.widgets.ProMTextField;
import org.processmining.subtrace.parameters.SubTraceFilteringParameters;

import javax.swing.*;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;
import java.awt.*;
import java.awt.event.KeyEvent;
import java.awt.event.KeyListener;
import java.util.ArrayList;
import java.util.List;

/**
 * Generates dialog for event classifier selection
 * Created by nsitnikov on 23/02/16.
 */
public class SubTraceEventClassifierStepDialog extends SubTraceDialog {
    private static final long serialVersionUID = -60087716353528968L;

    /**
     * The JPanel that allows the user to set (a subset of) the parameters.
     *
     * @param context UI context
     * @param input log to process
     * @param parameters parameters to update
     * @param logInfo computed log information
     */
    public SubTraceEventClassifierStepDialog(UIPluginContext context, XLog input, SubTraceFilteringParameters parameters, XLogInfo logInfo) {
        super(context, input, parameters, logInfo);
    }

    @Override
    protected JComponent generateList(final SubTraceFilteringParameters parameters) {
        JComponent jComponent = null;
        List<XEventClassifier> classifiers = new ArrayList<>(logInfo.getEventClassifiers());
        if (classifiers.size() == 0) {
            classifiers.add(XLogInfoImpl.STANDARD_CLASSIFIER);
        }

        // get selected indexes
        // TODO look for more elegant way
        List<Integer> selected = new ArrayList<>();
        for (int i=0; i < classifiers.size(); i++) {
            if (classifiers.get(i).equals(parameters.getClassifier())) {
                selected.add(i);
            }
        }

        final JList list = new JList<>(classifiers.toArray());
        list.clearSelection();
        if (selected.size() > 0)
            list.setSelectedIndices(ArrayUtils.toPrimitive(selected.toArray(new Integer[selected.size()])));
        list.addListSelectionListener(new ListSelectionListener() {
            public void valueChanged(ListSelectionEvent e) {
                parameters.setClassifier((XEventClassifier) list.getSelectedValue());
            }
        });
        jComponent = configureList(list,
                "Event classifier",
                "The event classes from selected classifier will be used on next screens.",
                false);

        return jComponent;
    }

    @Override
    protected JComponent generateRange(final SubTraceFilteringParameters parameters) {
        return null;
    }

    @Override
    protected JComponent generatePosition(SubTraceFilteringParameters parameters) {
        return null;
    }

    @Override
    protected JComponent generateExtraEvent(SubTraceFilteringParameters parameters) {
        return null;
    }

    @Override
    protected JComponent generateExtraName(SubTraceFilteringParameters parameters) {
        return null;
    }

    @Override
    protected JComponent generateOutputName(final SubTraceFilteringParameters parameters) {

        final ProMTextField textField = new ProMTextField(parameters.getOutgoingLogName());
        textField.setPreferredSize(new Dimension(100, 25));

        //TODO make it look nicer
        textField.addKeyListener(new KeyListener() {
            public void keyTyped(KeyEvent e) {
                parameters.setOutgoingLogName(textField.getText()); }
            public void keyPressed(KeyEvent e) {
                parameters.setOutgoingLogName(textField.getText()); }
            public void keyReleased(KeyEvent e) {
                parameters.setOutgoingLogName(textField.getText()); }
        });

        return textField;
    }
}
