package org.processmining.subtrace.dialogs;

import com.fluxicon.slickerbox.factory.SlickerFactory;
import org.apache.commons.lang3.ArrayUtils;
import org.deckfour.xes.classification.XEventClass;
import org.deckfour.xes.classification.XEventClasses;
import org.deckfour.xes.info.XLogInfo;
import org.deckfour.xes.model.XLog;
import org.processmining.contexts.uitopia.UIPluginContext;
import org.processmining.framework.util.ui.widgets.ProMTextField;
import org.processmining.subtrace.parameters.SubTraceFilteringParameters;

import javax.swing.*;
import javax.swing.event.DocumentEvent;
import javax.swing.event.DocumentListener;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;
import java.awt.*;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.KeyEvent;
import java.awt.event.KeyListener;
import java.text.NumberFormat;
import java.util.ArrayList;
import java.util.Collections;
import java.util.List;

/**
 * Gemerate dialog for start event selection
 * Created by nsitnikov on 23/02/16.
 */
public class SubTraceStartEventStepDialog extends SubTraceDialog {
    private static final long serialVersionUID = -62087716353523368L;

    /**
     * The JPanel that allows the user to set (a subset of) the parameters.
     *
     * @param context      UI context
     * @param input        log to process
     * @param parameters   parameters to update
     * @param logInfo      computed log info
     */
    public SubTraceStartEventStepDialog(UIPluginContext context, XLog input, SubTraceFilteringParameters parameters, XLogInfo logInfo) {
        super(context, input, parameters, logInfo);
    }

    @Override
    protected JComponent generateList(final SubTraceFilteringParameters parameters) {
        JComponent jComponent = null;
        XEventClasses eventClasses = logInfo.getEventClasses(parameters.getClassifier());
        if (eventClasses != null) {
            List<XEventClass> sortedEventClasses = new ArrayList<>(eventClasses.getClasses());
            Collections.sort(sortedEventClasses, new EventClassComparator());
            // TODO look for more elegant way
            List<Integer> selected = new ArrayList<>();
            for (int i=0; i < sortedEventClasses.size(); i++) {
                if (parameters.getStartEvents().contains(sortedEventClasses.get(i).getId())) {
                    selected.add(i);
                }
            }

            final JList list = new JList<>(sortedEventClasses.toArray());
            list.clearSelection();
            if (selected.size() > 0 )
                list.setSelectedIndices(ArrayUtils.toPrimitive(selected.toArray(new Integer[selected.size()])));
            list.addListSelectionListener(new ListSelectionListener() {
                public void valueChanged(ListSelectionEvent e) {
                    List<Object> selectedObjects = list.getSelectedValuesList();
                    List<String> startEvents = new ArrayList<>();
                    for (Object item : selectedObjects) {
                        startEvents.add(((XEventClass) item).getId());
                    }
                    parameters.setStartEvents(startEvents);
                }
            });
            jComponent = configureList(list,
                    "Start events",
                    "First fragment's activity should match selected event(s). Select none (alt-click to remove selection) to always start at the beginning of the trace.",
                    true);
        }
        ;

        return jComponent;
    }

    /**
     * Initialize range modifier
     */
    @Override
    protected JComponent generateRange(final SubTraceFilteringParameters parameters) {

        final ActionListener rangeListener = new ActionListener() {
            @Override
            public void actionPerformed(ActionEvent e) {
                if ("previous".equals(e.getActionCommand())) {
                    parameters.setStartRange(SubTraceFilteringParameters.RangeType.PREVIOUS);
                } else if ("current".equals(e.getActionCommand())) {
                    parameters.setStartRange(SubTraceFilteringParameters.RangeType.CURRENT);
                } else if ("next".equals(e.getActionCommand())) {
                    parameters.setStartRange(SubTraceFilteringParameters.RangeType.NEXT);
                }
            }
        };
        //Create the radio buttons.
        List<JRadioButton> buttons = new ArrayList<>();
        JRadioButton button = SlickerFactory.instance().createRadioButton("Previous activity");
        button.setActionCommand("previous");
        button.addActionListener(rangeListener);
        button.setSelected(parameters.getStartRange().equals(SubTraceFilteringParameters.RangeType.PREVIOUS));
        buttons.add(button);

        button = SlickerFactory.instance().createRadioButton("Matched activity");
        button.setActionCommand("current");
        button.addActionListener(rangeListener);
        button.setSelected(parameters.getStartRange().equals(SubTraceFilteringParameters.RangeType.CURRENT));
        buttons.add(button);

        button = SlickerFactory.instance().createRadioButton("Next activity");
        button.setActionCommand("next");
        button.addActionListener(rangeListener);
        button.setSelected(parameters.getStartRange().equals(SubTraceFilteringParameters.RangeType.NEXT));
        buttons.add(button);

        return configurePanel(buttons, "Choose which event to use:");
    }

    /**
     * Initialize position modifier
     */
    @Override
    protected JComponent generatePosition(final SubTraceFilteringParameters parameters) {
        final ActionListener positionListener = new ActionListener() {
            @Override
            public void actionPerformed(ActionEvent e) {
                if ("first".equals(e.getActionCommand())) {
                    parameters.setStartPositionType(SubTraceFilteringParameters.PositionType.FIRST);
                } else if ("last".equals(e.getActionCommand())) {
                    parameters.setStartPositionType(SubTraceFilteringParameters.PositionType.LAST);
                }
            }
        };
        //Create the radio buttons.
        List<JRadioButton> buttons = new ArrayList<>();
        JRadioButton button = SlickerFactory.instance().createRadioButton("First occurrence");
        button.setActionCommand("first");
        button.addActionListener(positionListener);
        button.setSelected(parameters.getStartPositionType().equals(SubTraceFilteringParameters.PositionType.FIRST));
        buttons.add(button);

        button = SlickerFactory.instance().createRadioButton("Last occurrence");
        button.setActionCommand("last");
        button.addActionListener(positionListener);
        button.setSelected(parameters.getStartPositionType().equals(SubTraceFilteringParameters.PositionType.LAST));
        buttons.add(button);

        return configurePanel(buttons, "Choose which event in trace to match:");
    }

    @Override
    protected JComponent generateExtraEvent(final SubTraceFilteringParameters parameters) {

        final ActionListener extraListener = new ActionListener() {
            @Override
            public void actionPerformed(ActionEvent e) {
                if ("keep".equals(e.getActionCommand())) {
                    parameters.setStartEdgeProcessing(SubTraceFilteringParameters.EdgeProcessingType.KEEP);
                } else if ("rename".equals(e.getActionCommand())) {
                    parameters.setStartEdgeProcessing(SubTraceFilteringParameters.EdgeProcessingType.RENAME);
                } else if ("add".equals(e.getActionCommand())) {
                    parameters.setStartEdgeProcessing(SubTraceFilteringParameters.EdgeProcessingType.ADD);
                }
            }
        };
        //Create the radio buttons.
        List<JRadioButton> buttons = new ArrayList<>();
        JRadioButton button = SlickerFactory.instance().createRadioButton("Keep name");
        button.setActionCommand("keep");
        button.addActionListener(extraListener);
        button.setSelected(parameters.getStartEdgeProcessing().equals(SubTraceFilteringParameters.EdgeProcessingType.KEEP));
        buttons.add(button);

        button = SlickerFactory.instance().createRadioButton("Rename activity");
        button.setActionCommand("rename");
        button.addActionListener(extraListener);
        button.setSelected(parameters.getStartEdgeProcessing().equals(SubTraceFilteringParameters.EdgeProcessingType.RENAME))
        ;
        buttons.add(button);

        button = SlickerFactory.instance().createRadioButton("Add activity");
        button.setActionCommand("add");
        button.addActionListener(extraListener);
        button.setSelected(parameters.getStartEdgeProcessing().equals(SubTraceFilteringParameters.EdgeProcessingType.RENAME));
        buttons.add(button);

        return configurePanel(buttons, "Choose edge event naming policy:");
    }

    @Override
    protected JComponent generateExtraName(final SubTraceFilteringParameters parameters) {
        final JTextField startEventName =  new JTextField(parameters.getStartEventName());
        startEventName.setHorizontalAlignment(SwingConstants.LEFT);

        DocumentListener textListener = new DocumentListener() {
            @Override
            public void insertUpdate(DocumentEvent e) {
                parameters.setStartEventName(startEventName.getText());
            }

            @Override
            public void removeUpdate(DocumentEvent e) {
                parameters.setStartEventName(startEventName.getText());
            }

            @Override
            public void changedUpdate(DocumentEvent e) {
                parameters.setStartEventName(startEventName.getText());
            }
        };

        startEventName.getDocument().addDocumentListener(textListener);
        return configureTextField(startEventName, "New event name:");
    }

    @Override
    protected JComponent generateOutputName(SubTraceFilteringParameters parameters) {
        return null;
    }
}