package org.processmining.subtrace.parameters;

import org.deckfour.xes.classification.XEventClassifier;
import org.deckfour.xes.info.impl.XLogInfoImpl;
import org.processmining.basicutils.parameters.impl.PluginParametersImpl;
import org.processmining.framework.plugin.events.NameChangeListener;

import java.util.ArrayList;
import java.util.List;

public class SubTraceFilteringParameters extends PluginParametersImpl implements NameChangeListener {

    public String getOutgoingLogName() {
        return this.outgoingLogName;
    }

    public XEventClassifier getClassifier() {
        return this.classifier;
    }

    public List<String> getStartEvents() {
        return this.startEvents;
    }

    public RangeType getStartRange() {
        return this.startRange;
    }

    public PositionType getStartPositionType() {
        return this.startPositionType;
    }

    public EdgeProcessingType getStartEdgeProcessing() {
        return this.startEdgeProcessing;
    }

    public String getStartEventName() {
        return this.startEventName;
    }

    public List<String> getEndEvents() {
        return this.endEvents;
    }

    public RangeType getEndRange() {
        return this.endRange;
    }

    public PositionType getEndPositionType() {
        return this.endPositionType;
    }

    public EdgeProcessingType getEndEdgeProcessing() {
        return this.endEdgeProcessing;
    }

    public String getEndEventName() {
        return this.endEventName;
    }

    public List<String> getRequiredEvents() {
        return this.requiredEvents;
    }

    public void setOutgoingLogName(String outgoingLogName) {
        this.outgoingLogName = outgoingLogName;
    }

    public void setClassifier(XEventClassifier classifier) {
        this.classifier = classifier;
    }

    public void setStartEvents(List<String> startEvents) {
        this.startEvents = startEvents;
    }

    public void setStartRange(RangeType startRange) {
        this.startRange = startRange;
    }

    public void setStartPositionType(PositionType startPositionType) {
        this.startPositionType = startPositionType;
    }

    public void setStartEdgeProcessing(EdgeProcessingType startEdgeProcessing) {
        this.startEdgeProcessing = startEdgeProcessing;
    }

    public void setStartEventName(String startEventName) {
        this.startEventName = startEventName;
    }

    public void setEndEvents(List<String> endEvents) {
        this.endEvents = endEvents;
    }

    public void setEndRange(RangeType endRange) {
        this.endRange = endRange;
    }

    public void setEndPositionType(PositionType endPositionType) {
        this.endPositionType = endPositionType;
    }

    public void setEndEdgeProcessing(EdgeProcessingType endEdgeProcessing) {
        this.endEdgeProcessing = endEdgeProcessing;
    }

    public void setEndEventName(String endEventName) {
        this.endEventName = endEventName;
    }

    public void setRequiredEvents(List<String> requiredEvents) {
        this.requiredEvents = requiredEvents;
    }

    public MatchingType getRequiredEventsMatching() {
        return requiredEventsMatching;
    }

    public void setRequiredEventsMatching(MatchingType requiredEventsMatching) {
        this.requiredEventsMatching = requiredEventsMatching;
    }

    public Integer getRequiredEventCount() {
        return requiredEventCount;
    }

    public void setRequiredEventCount(Integer requiredEventCount) {
        this.requiredEventCount = requiredEventCount;
    }

    public boolean isCalculateMetrics() {
        return calculateMetrics;
    }

    public void setCalculateMetrics(boolean calculateMetrics) {
        this.calculateMetrics = calculateMetrics;
    }

    public boolean equals(Object o) {
        if(!(o instanceof PluginParametersImpl)) {
            return false;
        } else {
            SubTraceFilteringParameters parameters = (SubTraceFilteringParameters) o;
            return
                this.getMessageLevel() == parameters.getMessageLevel() &&
                this.isTryConnections() == parameters.isTryConnections() &&
                this.startEvents.equals(parameters.getStartEvents()) &&
                this.startEdgeProcessing.equals(parameters.getStartEdgeProcessing()) &&
                this.startEventName.equals(parameters.getStartEventName()) &&
                this.startPositionType.equals(parameters.getStartPositionType()) &&
                this.startRange.equals(parameters.getStartRange()) &&
                this.endEvents.equals(parameters.getEndEvents()) &&
                this.endEdgeProcessing.equals(parameters.getEndEdgeProcessing()) &&
                this.endEventName.equals(parameters.getEndEventName()) &&
                this.endPositionType.equals(parameters.getEndPositionType()) &&
                this.endRange.equals(parameters.getEndRange()) &&
                this.requiredEvents.equals(parameters.getRequiredEvents());
        }
    }

    @Override
    public void nameChanged(String s) {
        System.out.println("new name" + s);
    }

    /**
     * Range type  of start event
     * ALL - all selected events should be present in selected fragment
     * ANY - at least one of events should be present in selected fragment
     * NONE - none of selected events should be present in selected fragment
     */
    public enum MatchingType {
        ALL,
        ANY,
        NONE
    }


    /**
     * Range type  of start event
     * EXACT - sub trace starts at specified event(s)
     * PREVIOUS - sub trace starts at event before specified events
     * NEXT - sub trace starts at event after specified event(s)
     */
    public enum RangeType {
        CURRENT,
        PREVIOUS,
        NEXT
    }

    /**
     * Rule to select of event if multiple are present in the trace
     * FIRST - selects first occurrence in trace
     * LAST - selects last occurrence to trace
     */
    public enum PositionType {
        FIRST,
        LAST
    }

    /**
     * Edge processing rule
     * KEEP - leave edge event as is
     * RENAME - rename edge event
     * ADD - add new activity before/after edge event
     */
    public enum EdgeProcessingType {
        KEEP,
        RENAME,
        ADD
    }

    private String outgoingLogName;

    private XEventClassifier classifier;

    private List<String> startEvents;
    private RangeType startRange;
    private PositionType startPositionType;
    private EdgeProcessingType startEdgeProcessing;
    private String startEventName;

    private List<String> endEvents;
    private RangeType endRange;
    private PositionType endPositionType;
    private EdgeProcessingType endEdgeProcessing;
    private String endEventName;

    private List<String> requiredEvents;
    private MatchingType requiredEventsMatching;
    private Integer requiredEventCount;

    private boolean calculateMetrics;


    /**
     * Create default parameter values.
     */
    public SubTraceFilteringParameters() {
        super();
        setDefaultValues();
    }

	public SubTraceFilteringParameters(SubTraceFilteringParameters parameters) {
		super(parameters);

        setDefaultValues(); // safety check

        this.setClassifier(parameters.getClassifier());
        this.setStartEvents(parameters.getStartEvents());
        this.setStartRange(parameters.getStartRange());
        this.setStartPositionType(parameters.getStartPositionType());
        this.setStartEventName(parameters.getStartEventName());
        this.setStartEdgeProcessing(parameters.getStartEdgeProcessing());
        this.setEndEvents(parameters.getEndEvents());
        this.setEndRange(parameters.getEndRange());
        this.setEndPositionType(parameters.getEndPositionType());
        this.setEndEventName(parameters.getEndEventName());
        this.setEndEdgeProcessing(parameters.getEndEdgeProcessing());
        this.setRequiredEvents(parameters.getRequiredEvents());
        this.setRequiredEventsMatching(parameters.getRequiredEventsMatching());
        this.setRequiredEventCount(parameters.getRequiredEventCount());
        this.setCalculateMetrics(parameters.isCalculateMetrics());
	}

    private void setDefaultValues() {
        this.setTryConnections(false);
        this.setStartEvents(new ArrayList<String>());
        this.setStartRange(RangeType.CURRENT);
        this.setStartPositionType(PositionType.LAST);
        this.setStartEventName("");
        this.setStartEdgeProcessing(EdgeProcessingType.KEEP);
        this.setEndEvents(new ArrayList<String>());
        this.setEndRange(RangeType.CURRENT);
        this.setEndPositionType(PositionType.FIRST);
        this.setEndEventName("");
        this.setEndEdgeProcessing(EdgeProcessingType.KEEP);
        this.setRequiredEvents(new ArrayList<String>());
        this.setClassifier(XLogInfoImpl.STANDARD_CLASSIFIER);
        this.setRequiredEventCount(1);
        this.setRequiredEventsMatching(MatchingType.ANY);
        this.setCalculateMetrics(false);
    }

    public String toString() {
        return "\nUsing '" + this.classifier.toString() + "' classifier filter fragments \n" +
                "find start event(s): " + (this.startEvents.size() == 0 ? "Any" : this.startEvents.toString()) +
                    " which is " + this.startPositionType.toString() + " in trace "+
                    "and choose " + this.startRange.toString() + "\n" +
                "find end event(s): " + (this.endEvents.size() == 0 ? "Any" : this.endEvents.toString()) +
                        " which is " + this.endPositionType.toString() + " in trace " +
                        " and choose " + this.endRange.toString() + " event\n" +
                this.requiredEventsMatching.toString() + " of required event(s): " + (this.requiredEvents.size() == 0 ? "None" : this.requiredEvents.toString()) + " should be matched in trace at least " +
                    this.requiredEventCount + " time(s)" +
                (this.startEdgeProcessing.equals(EdgeProcessingType.KEEP) ? "":
                        this.startEdgeProcessing.equals(EdgeProcessingType.RENAME) ? "\nrename start event to " + this.startEventName :
                            "\nprepend start event with event " + this.startEventName) +
                (this.endEdgeProcessing.equals(EdgeProcessingType.KEEP)? "":
                        this.endEdgeProcessing.equals(EdgeProcessingType.RENAME) ? "\nrename end event to " + this.endEventName :
                            "\nafter end event append event " + this.endEventName) +
                (this.calculateMetrics ? "\nand calculate trace metrics" : "");
    }

	
}
