package org.processmining.subtrace.plugins;

import com.thoughtworks.xstream.XStream;
import org.processmining.contexts.uitopia.annotations.UIExportPlugin;
import org.processmining.framework.plugin.PluginContext;
import org.processmining.framework.plugin.annotations.Plugin;
import org.processmining.framework.plugin.annotations.PluginLevel;
import org.processmining.framework.plugin.annotations.PluginQuality;
import org.processmining.framework.plugin.annotations.PluginVariant;
import org.processmining.subtrace.parameters.SubTraceFilteringParameters;

import java.io.*;
import java.nio.charset.Charset;

/**
 * Export SubTrace parameters to xml file
 * Created by nsitnikov on 24/02/16.
 */
@Plugin(name = "Fragment Extraction parameters export",
        quality = PluginQuality.Good,
        level = PluginLevel.NightlyBuild,
        returnLabels = {},
        returnTypes = {}, parameterLabels = {"SubTrace parameters", "File" })
@UIExportPlugin(description = "Fragment extraction parameters files", extension = "xml")
public class SubTraceParametersExportPlugin {

    @PluginVariant(variantLabel = "Fragment Extraction parameters export", requiredParameterLabels = { 0, 1 })
    public void export(PluginContext context, SubTraceFilteringParameters parameters, File file) throws IOException {
        XStream xstream = new XStream();
        xstream.alias("SubTraceFilter", SubTraceFilteringParameters.class);
        String xml = xstream.toXML(parameters);

        BufferedWriter output = null;
        try {
            output = new BufferedWriter(new FileWriter(file));
            output.write(xml);
        } catch ( IOException e ) {
            e.printStackTrace();
        } finally {
            if ( output != null ) output.close();
        }
    }

}
