package org.processmining.subtrace.plugins;

import com.thoughtworks.xstream.XStream;
import com.thoughtworks.xstream.XStreamException;
import org.apache.commons.io.IOUtils;
import org.processmining.contexts.uitopia.annotations.UIImportPlugin;
import org.processmining.framework.abstractplugins.AbstractImportPlugin;
import org.processmining.framework.plugin.PluginContext;
import org.processmining.framework.plugin.annotations.Plugin;
import org.processmining.framework.plugin.annotations.PluginLevel;
import org.processmining.framework.plugin.annotations.PluginQuality;
import org.processmining.subtrace.parameters.SubTraceFilteringParameters;

import javax.swing.filechooser.FileFilter;
import javax.swing.filechooser.FileNameExtensionFilter;
import java.beans.XMLDecoder;
import java.io.ByteArrayInputStream;
import java.io.InputStream;
import java.nio.charset.Charset;

/**
 * Import SubTrace parameters from xml file
 * Created by nsitnikov on 24/02/16.
 */

@Plugin(name = "Fragment Extraction parameters import",
        quality = PluginQuality.Good,
        level = PluginLevel.NightlyBuild,
        parameterLabels = { "Filename" },
        returnLabels = { "Fragment Extraction parameters" },
        returnTypes = { SubTraceFilteringParameters.class })
@UIImportPlugin(description = "Fragment extraction parameters files", extensions = { "xml" })
public class SubTraceParametersImportPlugin extends AbstractImportPlugin {
    protected FileFilter getFileFilter() {
        return new FileNameExtensionFilter("SubTrace parameters files", "xml");
    }

    protected Object importFromStream(PluginContext context, InputStream input, String filename, long fileSizeInBytes)
            throws Exception {

        SubTraceFilteringParameters parameters = new SubTraceFilteringParameters();

        XStream xstream = new XStream();
        xstream.alias("SubTraceFilter", SubTraceFilteringParameters.class);
        String xml = IOUtils.toString(input, Charset.forName("UTF8"));
        if (xml.contains("XMLDecoder")) {
            try {
                // try old format (XMLDecoder)
                XMLDecoder decoder = new XMLDecoder(new ByteArrayInputStream(xml.getBytes()));
                parameters = (SubTraceFilteringParameters) decoder.readObject();
            }
            catch (Exception e) {
                e.printStackTrace();
            }
        }
        else {
            try {
                parameters = (SubTraceFilteringParameters) xstream.fromXML(xml);
            } catch (XStreamException e) {
                e.printStackTrace();
            }
        }

        return parameters;
    }

}
